==> pvfs-1.1-9.tgz: pvfs-1.1/lib/pvfs_close.c <==
/* pvfs_close.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  rbross@parl.eng.clemson.edu  or
				 Matt Cettei mcettei@parl.eng.clemson.edu
/*/

/*
 * $Log: pvfs_close.c,v $
 * Revision 1.17  1998/07/23 21:54:06  rbross
 * using build_simple_jobs
 *
 * Revision 1.16  1998/07/14 18:40:47  rbross
 * *** empty log message ***
 *
 * Revision 1.15  1998/07/07 16:48:33  rbross
 * added much needed bounds checking on fds and on location of pathnames
 *
 * Revision 1.13  1998/02/16 18:57:44  rbross
 * Newt patches
 *
 * Revision 1.12  1997/10/27 21:08:37  pvfs
 * added license
 *
 * Revision 1.11  1997/07/10 18:34:51  walt
 * added fchmod fchown dup dup2 rmdir and fixed some minor bugs
 *
 * Revision 1.10  1997/06/04 17:22:27  mcettei
 * Will free pfds structure and set to NULL
 *
 * Revision 1.9  1997/06/03 20:12:06  mcettei
 * Sends messages to iods to close file instances
 *
 * Revision 1.8  1997/03/18 00:06:35  rbross
 * modified to just close sockets if file is opened in passive mode
 *
 * Revision 1.7  1997/02/11 22:09:51  mcettei
 * removed pvfs_fd union
 *
 * Revision 1.6  1997/02/07 15:26:16  rbross
 * minor bug fix
 *
 * Revision 1.5  1997/01/20 17:13:42  mcettei
 * *** empty log message ***
 *
 * Revision 1.4  1997/01/16 19:30:40  mcettei
 * uses debugging code
 *
 * Revision 1.3  1997/01/08 19:20:25  mcettei
 * closes file properly
 *
 * Revision 1.2  1996/12/06 20:52:52  mcettei
 * added header
 *
 *
/*/

static char pvfs_close_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_close.c,v 1.17 1998/07/23 21:54:06 rbross Exp $";

/* This file contains PVFS library call for close.   		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/
/* Determines if pvfs file and take file off 				*/
/* open list; also, close meta file								*/

#include <lib.h>

extern fdesc_p pfds[];
extern int errno;
extern sockset socks;
extern jlist_p active_p;

int pvfs_close(int fd)
{
	int i, s, ret, status=0;
	struct sockaddr saddr;
	mreq req;
	mack ack;
	ireq iodreq;
	iack iodack;

	if (fd < 0 || fd >= NR_OPEN || (pfds[fd] && pfds[fd]->fs == FS_RESV)) {
		errno = EBADF;
		return(-1);
	}

	/* Check to see what kind of file it is */
	if ((pfds[fd] && --pfds[fd]->fd.ref) || !pfds[fd] || pfds[fd]->fs==FS_UNIX)
	{
		if (pfds[fd] && !pfds[fd]->fd.ref) {
			if (pfds[fd]->part_p) free(pfds[fd]->part_p);
			if (pfds[fd]->dim_p)  free(pfds[fd]->dim_p);
			free(pfds[fd]);
		}
		pfds[fd] = NULL;
		return(syscall(SYS_close, fd));
	}

	/* Now need to send close directly to iods */
	iodreq.majik_nr   = IOD_MAJIK_NR;
	iodreq.type 		= IOD_CLOSE;
	iodreq.dsize		= 0;
	iodreq.req.close.f_ino	= pfds[fd]->fd.meta.u_stat.st_ino;
	iodreq.req.close.cap	= pfds[fd]->fd.cap;

	/* build job to send reqs and recv acks */
	if (build_simple_jobs(pfds[fd], &iodreq) < 0) {
		ERR("pvfs_ftruncate: build_simple_jobs failed\n");
		return(-1);
	}

	/* call do_job */
	while (!jlist_empty(active_p)) {
		if (do_jobs(active_p, &socks, -1) < 0) {
			ERR("pvfs_close: do_jobs failed\n");
			return(-1);
		}
	}

	/* check acks from iods */
	for (i=0; i < pfds[fd]->fd.meta.p_stat.pcount; i++) {
		if (pfds[fd]->fd.iod[i].ack.status) {
			ERR1("pvfs_ftruncate: non-zero status returned from iod %d\n", i);
			errno = pfds[fd]->fd.iod[i].ack.eno;
			status++;
		}
	}

	/* Close existing iod sockets */
	for (i = 0; i < pfds[fd]->fd.meta.p_stat.pcount; i++) {
		if (pfds[fd]->fd.iod[i].sock >= 0) {
			syscall(SYS_close, pfds[fd]->fd.iod[i].sock);
		}	
	}	
	for (i=0; i < pfds[fd]->fd.meta.p_stat.pcount; i++) {
		if (pfds[fd]->fd.iod[i].ack.status) {
			errno = pfds[fd]->fd.iod[i].ack.eno;
			return(pfds[fd]->fd.iod[i].ack.status);
		}
	}
	/* Prepare request for file system  */
	req.uid 		= getuid();
	req.gid 		= getgid();
	req.type 	= MGR_CLOSE;
	req.dsize 	= 0;
	req.req.close.meta	= pfds[fd]->fd.meta;

	if (!(pfds[fd]->fd.flag & O_GADD)) {
		/* Need to connect to mgr, not ready yet */
		saddr = pfds[fd]->fd.meta.mgr;
		if ((s = send_mreq_saddr(&saddr, &req, NULL, &ack)) < 0)
		{
			PERROR("send_mreq_saddr");
			return(-1);
		}
	}
	if (pfds[fd]->part_p) free(pfds[fd]->part_p);
	if (pfds[fd]->dim_p)  free(pfds[fd]->dim_p);
	free(pfds[fd]);
	pfds[fd] = NULL;
	syscall(SYS_close, fd);
	if (status) return(-1);
	return (0);
}

==> pvfs-1.3.2.tgz: pvfs-1.3.2/lib/pvfs_close.c <==
/* pvfs_close.c copyright (c) 1996 Rob Ross, all rights reserved.
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	Contact:  Rob Ross  rbross@parl.eng.clemson.edu  or
				 Matt Cettei mcettei@parl.eng.clemson.edu
/*/

/*
 * $Log: pvfs_close.c,v $
 * Revision 1.20  1999/06/17 15:36:46  rbross
 * bug fix WRT sending close messages to I/O daemons
 *
 * Revision 1.19  1999/02/19 00:30:05  rbross
 * cleanup
 *
 * Revision 1.18  1999/02/18 19:31:40  rbross
 * added sanity checking for fdesc and iod_info structures
 *
 * Revision 1.17  1998/07/23 21:54:06  rbross
 * using build_simple_jobs
 *
 * Revision 1.16  1998/07/14 18:40:47  rbross
 * *** empty log message ***
 *
 * Revision 1.15  1998/07/07 16:48:33  rbross
 * added much needed bounds checking on fds and on location of pathnames
 *
 * Revision 1.13  1998/02/16 18:57:44  rbross
 * Newt patches
 *
 * Revision 1.12  1997/10/27 21:08:37  pvfs
 * added license
 *
 * Revision 1.11  1997/07/10 18:34:51  walt
 * added fchmod fchown dup dup2 rmdir and fixed some minor bugs
 *
 * Revision 1.10  1997/06/04 17:22:27  mcettei
 * Will free pfds structure and set to NULL
 *
 * Revision 1.9  1997/06/03 20:12:06  mcettei
 * Sends messages to iods to close file instances
 *
 * Revision 1.8  1997/03/18 00:06:35  rbross
 * modified to just close sockets if file is opened in passive mode
 *
 * Revision 1.7  1997/02/11 22:09:51  mcettei
 * removed pvfs_fd union
 *
 * Revision 1.6  1997/02/07 15:26:16  rbross
 * minor bug fix
 *
 * Revision 1.5  1997/01/20 17:13:42  mcettei
 * *** empty log message ***
 *
 * Revision 1.4  1997/01/16 19:30:40  mcettei
 * uses debugging code
 *
 * Revision 1.3  1997/01/08 19:20:25  mcettei
 * closes file properly
 *
 * Revision 1.2  1996/12/06 20:52:52  mcettei
 * added header
 *
 *
/*/

static char pvfs_close_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_close.c,v 1.20 1999/06/17 15:36:46 rbross Exp $";

/* This file contains PVFS library call for close.   		*/
/* It will determine if file is UNIX or PVFS and then 	*/
/* make the proper request.										*/
/* Determines if pvfs file and take file off 				*/
/* open list; also, close meta file								*/

#include <lib.h>

extern fdesc_p pfds[];
extern int errno;
extern sockset socks;
extern jlist_p active_p;

int pvfs_close(int fd)
{
	int i, ret, status=0, myeno=0;
	struct sockaddr saddr;
	mreq req;
	mack ack;
	ireq iodreq;
	iack iodack;

	if (fd < 0 || fd >= NR_OPEN || (pfds[fd] && pfds[fd]->fs == FS_RESV)) {
		errno = EBADF;
		return(-1);
	}

	/* If we don't know anything about the FD, use the system call */
	if (!pfds[fd]) return(syscall(SYS_close, fd));

	/* Two cases here:
	 * - PVFS file with more references (first case)
	 * - UNIX file
	 *
	 * In both cases, we want to make sure to eliminate any data
	 * structures we held for the file.  Then we want to close the FD.
	 * For PVFS files, this will close the meta file, which will free the
	 * FD slot for later use.  For UNIX files, this will close the actual
	 * file, although there may be more open FD references to the same
	 * file.
	 */
	if ((pfds[fd] && --pfds[fd]->fd.ref) || pfds[fd]->fs==FS_UNIX)
	{
		if (pfds[fd] && !pfds[fd]->fd.ref) {
			if (pfds[fd]->part_p) free(pfds[fd]->part_p);
			if (pfds[fd]->dim_p)  free(pfds[fd]->dim_p);
			free(pfds[fd]);
		}
		pfds[fd] = NULL;
		return(syscall(SYS_close, fd));
	}

#ifdef STRICT_FDESC_CHECK
	if (do_fdesc_check(fd) < 0) {
		errno = EBADF;
		return(-1);
	}
#endif

	/* Now need to send close directly to iods */
	iodreq.majik_nr   = IOD_MAJIK_NR;
	iodreq.type 		= IOD_CLOSE;
	iodreq.dsize		= 0;
	iodreq.req.close.f_ino	= pfds[fd]->fd.meta.u_stat.st_ino;
	iodreq.req.close.cap	= pfds[fd]->fd.cap;

	/* build job to send reqs and recv acks to/from iods */
	if (!active_p) active_p = jlist_new();
	initset(&socks); /* clear out the socket set */

	if (build_simple_jobs(pfds[fd], &iodreq) < 0) {
		ERR("pvfs_close: build_simple_jobs failed\n");
		goto pvfs_close_try_manager;
	}

	/* call do_job */
	while (!jlist_empty(active_p)) {
		if (do_jobs(active_p, &socks, -1) < 0) {
			ERR("pvfs_close: do_jobs failed\n");
			goto pvfs_close_try_manager;
		}
	}

	/* look for errors from iods */
	for (i=0; i < pfds[fd]->fd.meta.p_stat.pcount; i++) {
		if (pfds[fd]->fd.iod[i].ack.status) {
			myeno = pfds[fd]->fd.iod[i].ack.eno;
			status = -1;
		}
	}

pvfs_close_try_manager:
	/* Prepare request for manager  */
	req.uid 		= getuid();
	req.gid 		= getgid();
	req.type 	= MGR_CLOSE;
	req.dsize 	= 0;
	req.req.close.meta = pfds[fd]->fd.meta;

	if (!(pfds[fd]->fd.flag & O_GADD)) {
		/* Need to connect to mgr, not ready yet */
		saddr = pfds[fd]->fd.meta.mgr;
		if (send_mreq_saddr(&saddr, &req, NULL, &ack) < 0)
		{
			myeno = errno;
			status = -1;
			PERROR("send_mreq_saddr");
		}
	}
#ifdef STRICT_FDESC_CHECK
	if (do_fdesc_check(fd) < 0) {
		errno = EBADF;
		return(-1);
	}
#endif

	/* Close existing iod sockets */
	for (i = 0; i < pfds[fd]->fd.meta.p_stat.pcount; i++) {
		if (pfds[fd]->fd.iod[i].sock >= 0) {
			syscall(SYS_close, pfds[fd]->fd.iod[i].sock);
		}	
	}	

	if (pfds[fd]->part_p) free(pfds[fd]->part_p);
	if (pfds[fd]->dim_p)  free(pfds[fd]->dim_p);
	free(pfds[fd]);
	pfds[fd] = NULL;
	syscall(SYS_close, fd); /* close the meta file */

	/* DON'T CLOSE THE MGR SOCKET!!!  IT CONFUSES THE MGRCOMM STUFF! */

	errno = myeno;
	return(status);
}
